﻿using Harmony;
using RimWorld;
using RimWorld.Planet;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using UnityEngine;
using Verse;
using Verse.AI;
using Verse.Sound;

namespace AppearanceClothes {
    [StaticConstructorOnStartup]
    class Main {
        static Main() {
            var harmony = HarmonyInstance.Create("com.tammybee.apparentclothes");
            harmony.PatchAll(Assembly.GetExecutingAssembly());

            AppearanceClothesPref.LoadPref();
        }
    }

    [HarmonyPatch(typeof(PawnGraphicSet))]
    [HarmonyPatch("ResolveAllGraphics")]
    public static class PawnGraphicSet_ResolveAllGraphics_Patch {

        [HarmonyPostfix]
        static void ApplyAppearanceBodyTypeDef(ref PawnGraphicSet __instance) {
            if (__instance.pawn.def.defName == "Human") {
                BodyTypeDef bodyType = __instance.pawn.GetAppearanceBodyTypeDef();
                __instance.nakedGraphic = GraphicDatabase.Get<Graphic_Multi>(bodyType.bodyNakedGraphicPath, ShaderDatabase.CutoutSkin, Vector2.one, __instance.pawn.story.SkinColor);
                __instance.rottingGraphic = GraphicDatabase.Get<Graphic_Multi>(bodyType.bodyNakedGraphicPath, ShaderDatabase.CutoutSkin, Vector2.one, PawnGraphicSet.RottingColor);
            }
        }
    }

    [HarmonyPatch(typeof(PawnGraphicSet))]
    [HarmonyPatch("ResolveApparelGraphics")]
    public static class PawnGraphicSet_ResolveApparelGraphics_Patch {

        [HarmonyPrefix]
        static bool Replace(ref PawnGraphicSet __instance) {
            __instance.ClearCache();
            __instance.apparelGraphics.Clear();

            if (__instance.pawn == null) {
                return false;
            }

            BodyTypeDef bodyType = __instance.pawn.GetAppearanceBodyTypeDef();
            CompAppearanceClothes compAppearanceClothes = __instance.pawn.GetComp<CompAppearanceClothes>();
            if (compAppearanceClothes == null || !compAppearanceClothes.ShowAppearanceClothes || compAppearanceClothes.AppearanceClothes == null) {
                //現在の装備を表示用画像に設定
                if (__instance.pawn.apparel != null) {
                    foreach (Apparel current in __instance.pawn.apparel.GetDirectlyHeldThings()) {
                        ApparelGraphicRecord item;
                        if (ApparelGraphicRecordGetter.TryGetGraphicApparel(current, bodyType, out item)) {
                            __instance.apparelGraphics.Add(item);
                        }
                    }
                }
            } else {
                //見た目装備を表示用画像に設定
                if (compAppearanceClothes.AppearanceClothes != null) {
                    for (int i=0;i < compAppearanceClothes.AppearanceClothes.Count;i++) {
                        Thing current = compAppearanceClothes.AppearanceClothes[i];
                        Apparel apparel = current as Apparel;
                        if (apparel != null && __instance.pawn.story != null) {
                            ApparelGraphicRecord item;
                            if (ApparelGraphicRecordGetter.TryGetGraphicApparel(apparel, bodyType, out item)) {
                                __instance.apparelGraphics.Add(item);
                            }
                        }
                    }
                }
            }

            return false;
        }
    }

    [HarmonyPatch(typeof(PawnRenderer))]
    [HarmonyPatch("BaseHeadOffsetAt")]
    public static class PawnRenderer_BaseHeadOffsetAt_Patch {

        [HarmonyPostfix]
        static void ApplyAppearanceBodyType(ref PawnRenderer __instance, Rot4 rotation, ref Vector3 __result) {
            Pawn pawn = Traverse.Create(__instance).Field("pawn").GetValue<Pawn>();
            CompAppearanceClothes comp = pawn.GetComp<CompAppearanceClothes>();
            if (comp != null && comp.ShowAppearanceClothes) {
                Vector2 headOffset = pawn.GetAppearanceBodyTypeDef().headOffset;
                switch (rotation.AsInt) {
                case 0:
                    __result = new Vector3(0f, 0f, headOffset.y);
                    break;
                case 1:
                    __result = new Vector3(headOffset.x, 0f, headOffset.y);
                    break;
                case 2:
                    __result = new Vector3(0f, 0f, headOffset.y);
                    break;
                case 3:
                    __result = new Vector3(-headOffset.x, 0f, headOffset.y);
                    break;
                default:
                    __result = Vector3.zero;
                    break;
                }
            }
        }
    }
}
