﻿using RimWorld;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Verse;

namespace AppearanceClothes {
    [StaticConstructorOnStartup]
    public static class Util {
        public static readonly List<string> bodyTypes = new List<string>() { "Male", "Female", "Thin", "Hulk", "Fat" };

        public static string GetName(this BodyTypeDef bodyType) {
            return bodyType.defName;
        }

        public static BodyTypeDef ToBodyTypeDef(this int value) {
            return DefDatabase<BodyTypeDef>.GetNamed(bodyTypes[value]);
        }

        public static BodyTypeDef GetAppearanceBodyTypeDef(this Pawn pawn) {
            if (pawn.story == null) {
                return BodyTypeDefOf.Male;
            }
            BodyTypeDef bodyType = pawn.story.bodyType;
            CompAppearanceClothes compAppearanceClothes = pawn.GetComp<CompAppearanceClothes>();
            if (compAppearanceClothes != null && compAppearanceClothes.ShowAppearanceClothes) {
                bodyType = compAppearanceClothes.appearanceBodyTypeDef;
            }
            return bodyType;
        }

        public static void Reorder<Type>(this List<Type> list, Type elem, int offset) {
            int num = list.IndexOf(elem);
            num += offset;
            if (num >= 0) {
                list.Remove(elem);
                list.Insert(num, elem);
            }
        }
    }
}
